﻿using MediatR;
using NetworkResurrector.Server.Wrapper.Services;
using System;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;

namespace NetworkResurrector.Api.Application.Queries
{
    public class GetSystemVersion
    {
        public class Query : IRequest<Model> { }

        public class Model
        {
            public ServiceVersion Api { get; set; }
            public ServiceVersion Server { get; set; }
        }

        public record ServiceVersion(string Version, DateTime LastReleaseDate);

        public class QueryHandler : IRequestHandler<Query, Model>
        {
            private readonly IResurrectorService _resurrectorService;

            public QueryHandler(IResurrectorService resurrectorService)
            {
                _resurrectorService=resurrectorService;
            }

            public async Task<Model> Handle(Query request, CancellationToken cancellationToken)
            {
                var serverVersion = await _resurrectorService.GetServiceVersion();
                var apiVersion = GetApiVersion();

                var result = new Model
                {
                    Api = apiVersion,
                    Server = new ServiceVersion(serverVersion.Version, serverVersion.LastReleaseDate)
                };

                return result;
            }

            private ServiceVersion GetApiVersion()
            {
                var version = Environment.GetEnvironmentVariable("APP_VERSION");
                var appDate = Environment.GetEnvironmentVariable("APP_DATE");

                if (string.IsNullOrEmpty(version))
                    version = Assembly.GetEntryAssembly().GetCustomAttribute<AssemblyInformationalVersionAttribute>().InformationalVersion;

                if (!DateTime.TryParse(appDate, out var lastReleaseDate))
                {
                    var location = Assembly.GetExecutingAssembly().Location;
                    lastReleaseDate = File.GetLastWriteTime(location);
                }

                var result = new ServiceVersion(version, lastReleaseDate);
                return result;
            }
        }
    }
}
